/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */

package jalview.fts.service.uniprot;

import jalview.fts.api.FTSData;
import jalview.fts.api.FTSDataColumnI;
import jalview.fts.api.FTSRestClientI;
import jalview.fts.core.FTSRestClient;
import jalview.fts.core.FTSRestRequest;
import jalview.fts.core.FTSRestResponse;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Objects;

import javax.ws.rs.core.MediaType;

import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.api.client.config.ClientConfig;
import com.sun.jersey.api.client.config.DefaultClientConfig;

public class UniProtFTSRestClient extends FTSRestClient
{
  private static FTSRestClientI instance = null;

  public static final String UNIPROT_SEARCH_ENDPOINT = "http://www.uniprot.org/uniprot/?";

  @Override
  public FTSRestResponse executeRequest(FTSRestRequest uniportRestRequest)
  {
    ClientConfig clientConfig = new DefaultClientConfig();
    Client client = Client.create(clientConfig);

    String wantedFields = getDataColumnsFieldsAsCommaDelimitedString(uniportRestRequest
            .getWantedFields());
    int responseSize = (uniportRestRequest.getResponseSize() == 0) ? getDefaultResponsePageSize()
            : uniportRestRequest.getResponseSize();

    String query = uniportRestRequest.getFieldToSearchBy()
            .equalsIgnoreCase("Search All") ? uniportRestRequest
            .getSearchTerm()
            : uniportRestRequest.getFieldToSearchBy() + ":"
                    + uniportRestRequest.getSearchTerm();

    // + (uniportRestRequest.isAllowUnpublishedEntries() ? ""
    // : " AND status:REL");
    // System.out.println(">>>>> Query : " + query);
    // System.out.println(">>>>> Columns : " + wantedFields);
    WebResource webResource = null;
    webResource = client.resource(UNIPROT_SEARCH_ENDPOINT)
            .queryParam("format", "tab")
            .queryParam("columns", wantedFields)
            .queryParam("limit", String.valueOf(responseSize))
            .queryParam("query", query);
    // Execute the REST request
    ClientResponse clientResponse = webResource
            .accept(MediaType.TEXT_PLAIN).get(ClientResponse.class);
    String uniProtTabDelimittedResponseString = clientResponse
            .getEntity(String.class);
    // Make redundant objects eligible for garbage collection to conserve
    // memory
    clientResponse = null;
    client = null;
    // System.out.println(">>>>> response : "
    // + uniProtTabDelimittedResponseString);
    return parseUniprotResponse(uniProtTabDelimittedResponseString,
            uniportRestRequest);

  }

  public FTSRestResponse parseUniprotResponse(
          String uniProtTabDelimittedResponseString,
          FTSRestRequest uniprotRestRequest)
  {
    FTSRestResponse searchResult = new FTSRestResponse();
    List<FTSData> result = null;
    String[] foundDataRow = uniProtTabDelimittedResponseString.split("\n");
    if (foundDataRow != null && foundDataRow.length > 0)
    {
      result = new ArrayList<FTSData>();
      String titleRow = getDataColumnsFieldsAsTabDelimitedString(uniprotRestRequest
              .getWantedFields());
      // System.out.println(">>>>Title row : " + titleRow);
      for (String dataRow : foundDataRow)
      {
        if (dataRow.equalsIgnoreCase(titleRow))
        {
          // System.out.println(">>>>>>>>>> matched!!!");
          continue;
        }
        // System.out.println(dataRow);
        result.add(getFTSData(dataRow, uniprotRestRequest));
      }
      searchResult.setNumberOfItemsFound(result.size());
      searchResult.setSearchSummary(result);
    }
    return searchResult;
  }

  public static FTSData getFTSData(String tabDelimittedDataStr,
          FTSRestRequest request)
  {
    String primaryKey = null;

    Object[] summaryRowData;

    Collection<FTSDataColumnI> diplayFields = request.getWantedFields();
    int colCounter = 0;
    summaryRowData = new Object[diplayFields.size()];
    String[] columns = tabDelimittedDataStr.split("\t");
    for (FTSDataColumnI field : diplayFields)
    {
      try
      {
        String fieldData = columns[colCounter];
        if (field.isPrimaryKeyColumn())
        {
          primaryKey = fieldData;
          summaryRowData[colCounter++] = primaryKey;
        }
        else if (fieldData == null || fieldData.isEmpty())
        {
          summaryRowData[colCounter++] = null;
        }
        else
        {
          try
          {
            summaryRowData[colCounter++] = (field.getDataColumnClass() == Integer.class) ? Integer
                    .valueOf(fieldData)
                    : (field.getDataColumnClass() == Double.class) ? Double
                            .valueOf(fieldData) : fieldData;
          } catch (Exception e)
          {
            e.printStackTrace();
              System.out.println("offending value:" + fieldData);
          }
        }
      } catch (Exception e)
      {
        // e.printStackTrace();
      }
    }

    final String primaryKey1 = primaryKey;

    final Object[] summaryRowData1 = summaryRowData;
    return new FTSData()
    {
      @Override
      public Object[] getSummaryData()
      {
        return summaryRowData1;
      }

      @Override
      public Object getPrimaryKey()
      {
        return primaryKey1;
      }

      /**
       * Returns a string representation of this object;
       */
      @Override
      public String toString()
      {
        StringBuilder summaryFieldValues = new StringBuilder();
        for (Object summaryField : summaryRowData1)
        {
          summaryFieldValues.append(
                  summaryField == null ? " " : summaryField.toString())
                  .append("\t");
        }
        return summaryFieldValues.toString();
      }

      /**
       * Returns hash code value for this object
       */
      @Override
      public int hashCode()
      {
        return Objects.hash(primaryKey1, this.toString());
      }
    };
  }


  public static FTSRestClientI getInstance()
  {
    if (instance == null)
    {
      instance = new UniProtFTSRestClient();
    }
    return instance;
  }

  @Override
  public String getColumnDataConfigFileName()
  {
    return "/fts/uniprot_data_columns.txt";
  }

}
